# Old school OpenGL
#

import pyglet
import pyglet.gl as pgl
from math import pi, cos, sin

# Define a simple function to create ctypes arrays of floats:
def vec(*args):
    return (pgl.GLfloat * len(args))(*args)


class Torus(object):
    def __init__(self, radius, inner_radius, slices, inner_slices):
        # Create the vertex and normal arrays.
        vertices = []
        normals = []

        u_step = 2 * pi / (slices - 1)
        v_step = 2 * pi / (inner_slices - 1)
        u = 0.
        for i in range(slices):
            cos_u = cos(u)
            sin_u = sin(u)
            v = 0.
            for j in range(inner_slices):
                cos_v = cos(v)
                sin_v = sin(v)

                d = (radius + inner_radius * cos_v)
                x = d * cos_u
                y = d * sin_u
                z = inner_radius * sin_v

                nx = cos_u * cos_v
                ny = sin_u * cos_v
                nz = sin_v

                vertices.extend([x, y, z])
                normals.extend([nx, ny, nz])
                v += v_step
            u += u_step

        vertices = vec(*vertices)
        normals = vec(*normals)

        # Create a list of triangle indices.
        indices = []
        for i in range(slices - 1):
            for j in range(inner_slices - 1):
                p = i * inner_slices + j
                indices.extend([p, p + inner_slices, p + inner_slices + 1])
                indices.extend([p,  p + inner_slices + 1, p + 1])
        indices = (pgl.GLuint * len(indices))(*indices)

        # Compile a display list
        self.list = pgl.glGenLists(1)
        pgl.glNewList(self.list, pgl.GL_COMPILE)

        pgl.glPushClientAttrib(pgl.GL_CLIENT_VERTEX_ARRAY_BIT)
        pgl.glEnableClientState(pgl.GL_VERTEX_ARRAY)
        pgl.glEnableClientState(pgl.GL_NORMAL_ARRAY)
        pgl.glVertexPointer(3,pgl.GL_FLOAT, 0, vertices)
        pgl.glNormalPointer(pgl.GL_FLOAT, 0, normals)
        pgl.glDrawElements(pgl.GL_TRIANGLES, len(indices),pgl.GL_UNSIGNED_INT, indices)
        pgl.glPopClientAttrib()

        pgl.glEndList()

    def draw(self):
        pgl.glCallList(self.list)


class MyWindow(pyglet.window.Window):

    def __init__(self):
        super(MyWindow, self).__init__()
        # One-time setup

        pgl.glClearColor(1, 1, 1, 1)
        pgl.glColor3f(1, 0, 0)
        pgl.glEnable(pgl.GL_DEPTH_TEST)
        pgl.glEnable(pgl.GL_CULL_FACE)

        pgl.glEnable(pgl.GL_LIGHTING)
        pgl.glEnable(pgl.GL_LIGHT0)


        pgl.glLightfv(pgl.GL_LIGHT0,pgl.GL_POSITION, vec(.5, .5, 1, 0))
        pgl.glLightfv(pgl.GL_LIGHT0,pgl.GL_SPECULAR, vec(.5, .5, 1, 1))
        pgl.glLightfv(pgl.GL_LIGHT0,pgl.GL_DIFFUSE, vec(1, 1, 1, 1))

        pgl.glMaterialfv(pgl.GL_FRONT_AND_BACK,pgl.GL_AMBIENT_AND_DIFFUSE, vec(0.5, 0, 0.3, 1))
        pgl.glMaterialfv(pgl.GL_FRONT_AND_BACK,pgl.GL_SPECULAR, vec(1, 1, 1, 1))
        pgl.glMaterialf(pgl.GL_FRONT_AND_BACK,pgl.GL_SHININESS, 50)


        self._torus = Torus(1, 0.3, 50, 30)
        self._rx = self._ry = self._rz = 0
        pyglet.clock.schedule(self._update)

    def on_draw(self):
        self.clear()
        pgl.glLoadIdentity()
        pgl.glTranslatef(0, 0, -4)
        pgl.glRotatef(self._rz, 0, 0, 1)
        pgl.glRotatef(self._ry, 0, 1, 0)
        pgl.glRotatef(self._rx, 1, 0, 0)
        self._torus.draw()

    # NB. We need setu the projection in the on_resize handler, since the
    # default pyglet on_resize handler sets up a different
    # projection.
    def on_resize(self, width, height):
        pgl.glViewport(0, 0, width, height)
        pgl.glMatrixMode(pgl.GL_PROJECTION)
        pgl.glLoadIdentity()
        pgl.gluPerspective(60., width / float(height), .1, 1000.)
        pgl.glMatrixMode(pgl.GL_MODELVIEW)

    def _update(self, dt):
        self._rx += dt * 1
        self._ry += dt * 80
        self._rz += dt * 30
        self._rx %= 360
        self._ry %= 360
        self._rz %= 360


if __name__ == "__main__":
    win = MyWindow()
    pyglet.app.run()

